%% Example: Basic Frequency Domain Tradeoffs
% This file considers a simple feedback system with G(s) = 1/s and
% K(s)=Kp.  Two different proportional gains are considered corresponding
% to slow and fast closed-loop response.

%% Plant
Gnum = 1;
Gden = [1 0];
G = tf(Gnum,Gden);

%% Frequency vector
Nw = 1000;
w = logspace(-2,2,Nw)';

%% Limits for Bode plot axes
xmin = w(1);
xmax = w(end);
ymin = -40;
ymax = 40;

%% Specify reference and noise signals for simulations
N = 1000;
Tf = 10;
tData = linspace(0,Tf,N);
tData = tData(:);

% Slow ramp up and down
idx1 = tData<2;
idx2 = tData>=2 & tData<=8;
idx3 = tData>8;
rData = zeros(N,1);
rData(idx1) = 1-cos(pi*tData(idx1)/2);
rData(idx2) = 2;
rData(idx3) = 1+cos( pi*(tData(idx3)-8)/2 );
rData = [tData rData];

% Reset random number generator
% This allows the results\ for the random signal to be repeatable.
stream = RandStream.getGlobalStream;
reset(stream);

% Random noise
nData = 0.5*randn(N,1);
nData = [tData nData];


%% Generate results for each proportional controller
for i=1:2    
    %% Controller K and Loop L=GK
    if i==1
        Kp = 1;
    else
        Kp = 10;
    end
    L = G*Kp;
    
    %% Bode Magnitudes of S, T, and L
    Lmag = bode(L,w);
    Lmag = Lmag(:);
    LmagdB = 20*log10(Lmag);
    
    T = feedback(L,1);
    Tmag = bode(T,w);
    Tmag = Tmag(:);
    TmagdB = 20*log10(Tmag);    
    
    S = feedback(1,L);
    Smag = bode(S,w);
    Smag = Smag(:);
    SmagdB = 20*log10(Smag);        
    
    % Plot |S|, |T| and |L|
    figure(2*i-1);
    clf
    semilogx(w,SmagdB,'b',w,TmagdB,'r-.',w,LmagdB,'k:');
    axis([xmin xmax ymin ymax]);
    ylabel('Magnitude, dB');
    xlabel('Frequency, rad/sec');
    legend('|S|','|T|','|L|','Location','Best');
    set(gca,'YTick',-40:20:40);
    set(gca, 'LooseInset', get(gca, 'TightInset'));    
    grid on;
    
    % Label Frequency bands
    yP = -30;
    phL = patch([xmin Kp/3 Kp/3 xmin xmin],[ymin ymin yP yP ymin],'b');
    phM = patch(Kp*[1/3 3 3 1/3 1/3],[ymin ymin yP yP ymin],'k');
    phH = patch([3*Kp xmax xmax 3*Kp 3*Kp],[ymin ymin yP yP ymin],'r');
    set([phL phM phH],'FaceAlpha',0.1,'LineStyle','none');
    text(Kp/10, -35,'Low','FontSize',14)
    text(Kp, -35,'Mid','HorizontalAlignment','Center','FontSize',14)
    text(4*Kp, -35,'High','FontSize',14)
    
    %% Simulate Closed Loop with Reference and Noise    
    sim('TradeoffsExample');
        
    figure(2*i);
    clf;
    plot(t,r,'b',t,y,'r-.')
    grid
    xlabel('Time, sec');
    ylabel('Output, y');
    legend('Reference, r','Output, y','Location','Best');
    axis([0 Tf -0.5 2.5]);
    set(gca, 'LooseInset', get(gca, 'TightInset'));   
    
end

%% Save Plots
if exist('garyfyFigure','file') && exist('myprint','file')
    % Code to clean up and save figure.
    % These functions are not in the standard Matlab release.
    figure(1)
    garyfyFigure
    myprint(1,'TradeoffsSTL1')
    
    figure(2)
    garyfyFigure
    myprint(2,'TradeoffsSim1')
    
    figure(3)
    garyfyFigure
    myprint(3,'TradeoffsSTL10')
    
    figure(4)
    garyfyFigure
    myprint(4,'TradeoffsSim10')
end
